import requestor from '../src'
import jQuery from 'jquery'

describe('requestor', function () {
  sinon.stub(jQuery, 'ajax')

  const SESSION_EXPIRED = 403
  const SUCCEED = 200
  const FAILED = 500

  var ajax

  var config = {
    resultCode: {
      SESSION_EXPIRED,
      SUCCEED
    },
    token: {
      checkBeforeSend: false,
      addToHeader: false
    },
    onSessionExpired: sinon.spy(),
    alert: sinon.spy()
  }

  beforeEach(function () {
    ajax = requestor(jQuery, config)
    config.onSessionExpired.reset()
    config.alert.reset()
  })

  it('create instance ok', function () {
    expect(ajax).to.not.be.null
    expect(ajax.request).to.be.a('function')
    expect(ajax.requestAPI).to.be.a('function')
  })

  it('requestAPI() normal', function () {
    let promise = ajax.requestAPI({url: '/api'}, {query: {a: 'a'}})

    expect(promise instanceof Promise).to.be.true
    expect(jQuery.ajax.calledWithMatch({url: '/api?a=a'})).to.be.true
  })

  // it('requestAPI() should add token to headers', function () {
  //   let TOKEN = '#@$#@$dde23'
  //   let oldCookie = document.cookie
  //   document.cookie += '&cookie=' + TOKEN
  //
  //   ajax.requestAPI('/url')
  //   document.cookie = oldCookie
  //
  //   expect(jQuery.ajax.calledWithMatch({headers: {'X-Token': TOKEN}})).to.be.true
  // })

  describe('request()', function () {
    afterEach(function () {
      jQuery.ajax.reset()
    })

    it('invoke success callback if response code be SUCCEED', function (done) {
      let response = {
        code: SUCCEED,
        result: {
          list: [1, 2, 3]
        }
      }

      jQuery.ajax.returns({
        then: function (okCb) {
          setTimeout(function () {
            okCb(response)
          }, 0)
        }
      })

      let failedCb = sinon.spy()

      ajax.request({url: '/url'}).then(function (result) {
        expect(result).to.equal(response.result)
        expect(failedCb.called).to.be.false
        done()
      }, failedCb)
    })

    it('invoke failed callback if response code be FAILED', function (done) {
      let response = {
        code: FAILED,
        result: {}
      }

      jQuery.ajax.returns({
        then: function (okCb) {
          setTimeout(function () {
            okCb(response)
          }, 0)
        }
      })

      let succCb = sinon.spy()

      ajax.request({url: '/url'}).then(succCb, function (res) {
        expect(res).to.equal(response)
        done()
        expect(succCb.called).to.be.false
      })
    })

    it('invoke failed callback if ajax failed', function (done) {
      const ajaxError = 'ajaxError'

      jQuery.ajax.returns({
        then: function (_, failedCb) {
          setTimeout(function () {
            failedCb(ajaxError)
          }, 0)
        }
      })

      let succCb = sinon.spy()

      ajax.request({url: '/url'}).then(succCb, function (error) {
        expect(error).to.equal(ajaxError)
        expect(succCb.called).to.be.false
        done()
      })
    })

    it('invoke onSessionExpired if response code be SESSION_EXPIRED', function (done) {
      let response = {
        code: SESSION_EXPIRED
      }

      jQuery.ajax.returns({
        then: function (okCb) {
          setTimeout(function () {
            okCb(response)
          }, 0)
        }
      })

      let succCb = sinon.spy()

      ajax.request({url: '/url'}).then(succCb, function () {
        expect(config.onSessionExpired.called).to.be.true
        expect(succCb.called).to.be.false
        done()
      })
    })

    it('alert message if presents in response and reqeust failed', function (done) {
      let response = {
        code: FAILED,
        msg: 'error msg'
      }

      jQuery.ajax.returns({
        then: function (okCb) {
          setTimeout(function () {
            okCb(response)
          }, 0)
        }
      })

      ajax.request({url: '/url'}).then(null, function () {
        // 消息是通过异步显示的，所以这里也要异步断言
        setTimeout(function () {
          expect(config.alert.calledWith(response.msg)).to.be.true
          done()
        })
      })
    })

    it('should not alert message if response code be SUCCEED', function (done) {
      let response = {
        code: SUCCEED,
        msg: 'some msg'
      }

      jQuery.ajax.returns({
        then: function (okCb) {
          setTimeout(function () {
            okCb(response)
          }, 0)
        }
      })

      ajax.request({url: '/url'}).then(function () {
        setTimeout(function () {
          expect(config.alert.called).to.be.false
          done()
        })
      })
    })
  })
})
