import util from '../src/util'

describe('util', function () {
  describe('normalizeUrl(url, query, option)', function () {
    it('return origin url if given url starts with protocol', function () {
      let url = 'http://a.com'
      expect(util.normalizeUrl(url)).to.equal(url)
    })

    it('append given host and port', function () {
      let url = '/abc'
      let host = 'http://www.a.com'
      let port = '8080'

      let fullUrl = util.normalizeUrl(url, null, {host, port})

      expect(fullUrl).to.equal(host + ':' + port + url)
    })

    it('append given query parameters', function () {
      let url = '/abc'
      let host = 'http://www.a.com'
      let query = {a: 3, b: '4'}

      let fullUrl = util.normalizeUrl(url, query, { host })

      expect(fullUrl).to.equal(host + url + '?a=' + query.a + '&b=' + query.b)
    })

    it('append more query parameters to url which has some query yet', function () {
      let url = '/abc?a=3'
      let host = 'http://www.a.com'
      let query = {b: '4'}

      let fullUrl = util.normalizeUrl(url, query, { host })

      expect(fullUrl).to.equal(host + url + '&b=' + query.b)
    })

    it('append base', function () {
      let url = '/abc?a=3'
      let base = '/base'

      let fullUrl = util.normalizeUrl(url, null, { base })

      expect(fullUrl).to.equal(base + url)
    })
  })

  describe('stringifyData(data)', function () {
    it('normal', function () {
      let data  ={
        n: 1,
        b: true,
        o: {a: 1},
        a: [1, 2, 3]
      }

      util.stringifyData(data)

      expect(data.b).to.equal(true)
      expect(data.n).to.equal(1)
      expect(data.o).to.equal('{"a":1}')
      expect(data.a).to.equal('[1,2,3]')
    })
  })

  describe('mergeAjaxOption(...options)', function () {
    it('merge option normal', function () {
      let option1 = {a: '1', c: '3'}
      let option2 = {a: '2', b: '2'}

      let result = util.mergeAjaxOption(option1, option2)

      expect(result.a).to.equal(option2.a)
      expect(result.b).to.equal(option2.b)
      expect(result.c).to.equal(option1.c)
    })

    it('merge query normal', function () {
      let query1 = {a: '1', c: '3'}
      let query2 = {a: '2', b: '2'}

      let result = util.mergeAjaxOption({query: query1}, {query: query2})

      expect(result.query.a).to.equal(query2.a)
      expect(result.query.b).to.equal(query2.b)
      expect(result.query.c).to.equal(query1.c)
    })

    it('merge data normal', function () {
      let data1 = {a: '1', c: '3'}
      let data2 = {a: '2', b: '2'}

      let result = util.mergeAjaxOption({data: data1}, {data: data2})

      expect(result.data.a).to.equal(data2.a)
      expect(result.data.b).to.equal(data2.b)
      expect(result.data.c).to.equal(data1.c)
    })

    it('merge headers normal', function () {
      let headers1 = {a: '1', c: '3'}
      let headers2 = {a: '2', b: '2'}

      let result = util.mergeAjaxOption({headers: headers1}, {headers: headers2})

      expect(result.headers.a).to.equal(headers2.a)
      expect(result.headers.b).to.equal(headers2.b)
      expect(result.headers.c).to.equal(headers1.c)
    })

    it('merge more than two option ok', function () {
      let option1 = {a: '1', c: '3'}
      let option2 = {a: '2', b: '2'}
      let option3 = {a: '4', d: '5'}

      let result = util.mergeAjaxOption(option1, option2, option3)

      expect(result.a).to.equal(option3.a)
      expect(result.b).to.equal(option2.b)
      expect(result.c).to.equal(option1.c)
      expect(result.d).to.equal(option3.d)
    })
  })

})
